@@ -53,7 +53,7 @@ bt_hex(const void *buf, size_t len)
     str = hexbufs[curbuf++];
     curbuf %= ARRAY_SIZE(hexbufs);
 
-    len = min(len, (sizeof(hexbufs[0]) - 1) / 2);
+    len = MIN(len, (sizeof(hexbufs[0]) - 1) / 2);
 
     for (i = 0; i < len; i++) {
         str[i * 2] = hex[b[i] >> 4];
@@ -588,7 +588,7 @@ int
 bt_rand(void *buf, size_t len)
 {
     int rc;
-    rc = ble_hs_hci_util_rand(buf, len);
+    rc = ble_hs_hci_rand(buf, len);
     if (rc != 0) {
         return -1;
     }
