@@ -49,6 +49,15 @@
 #include "ble_hs_priv.h"
 
 #if NIMBLE_BLE_CONNECT
+
+#ifndef min
+#define min(a, b) ((a) < (b) ? (a) : (b))
+#endif
+
+#ifndef max
+#define max(a, b) ((a) > (b) ? (a) : (b))
+#endif
+
 #if NIMBLE_BLE_SM
 
 /** Procedure timeout; 30 seconds. */
@@ -261,7 +270,7 @@ ble_sm_gen_pair_rand(uint8_t *pair_rand)
     }
 #endif
 
-    rc = ble_hs_hci_util_rand(pair_rand, 16);
+    rc = ble_hs_hci_rand(pair_rand, 16);
     if (rc != 0) {
         return rc;
     }
@@ -282,7 +291,7 @@ ble_sm_gen_ediv(struct ble_sm_master_id *master_id)
     }
 #endif
 
-    rc = ble_hs_hci_util_rand(&master_id->ediv, sizeof master_id->ediv);
+    rc = ble_hs_hci_rand(&master_id->ediv, sizeof master_id->ediv);
     if (rc != 0) {
         return rc;
     }
@@ -303,7 +312,7 @@ ble_sm_gen_master_id_rand(struct ble_sm_master_id *master_id)
     }
 #endif
 
-    rc = ble_hs_hci_util_rand(&master_id->rand_val, sizeof master_id->rand_val);
+    rc = ble_hs_hci_rand(&master_id->rand_val, sizeof master_id->rand_val);
     if (rc != 0) {
         return rc;
     }
@@ -325,7 +334,7 @@ ble_sm_gen_ltk(struct ble_sm_proc *proc, uint8_t *ltk)
     }
 #endif
 
-    rc = ble_hs_hci_util_rand(ltk, proc->key_size);
+    rc = ble_hs_hci_rand(ltk, proc->key_size);
     if (rc != 0) {
         return rc;
     }
@@ -350,7 +359,7 @@ ble_sm_gen_csrk(struct ble_sm_proc *proc, uint8_t *csrk)
     }
 #endif
 
-    rc = ble_hs_hci_util_rand(csrk, 16);
+    rc = ble_hs_hci_rand(csrk, 16);
     if (rc != 0) {
         return rc;
     }
@@ -564,7 +573,8 @@ ble_sm_persist_keys(struct ble_sm_proc *proc)
     ble_hs_unlock();
 
     if (identity_ev) {
-        ble_gap_identity_event(proc->conn_handle);
+        /* Use peer_addr since it does have proper addr type (i.e. 0/1, not 2/3) */
+        ble_gap_identity_event(proc->conn_handle, &peer_addr);
     }
 
     authenticated = proc->flags & BLE_SM_PROC_F_AUTHENTICATED;
@@ -894,7 +904,8 @@ ble_sm_chk_repeat_pairing(uint16_t conn_handle,
 }
 
 void
-ble_sm_process_result(uint16_t conn_handle, struct ble_sm_result *res)
+ble_sm_process_result(uint16_t conn_handle, struct ble_sm_result *res,
+                      bool tx_fail)
 {
     struct ble_sm_proc *prev;
     struct ble_sm_proc *proc;
@@ -927,12 +938,18 @@ ble_sm_process_result(uint16_t conn_handle, struct ble_sm_result *res)
             }
         }
 
-        if (res->sm_err != 0) {
+        if (res->sm_err != 0 && tx_fail) {
             ble_sm_pair_fail_tx(conn_handle, res->sm_err);
         }
 
         ble_hs_unlock();
 
+        if (res->enc_cb &&
+            res->app_status != BLE_HS_ENOTCONN) {
+            /* Do not send this event on broken connection */
+            ble_gap_pairing_complete_event(conn_handle, res->sm_err);
+        }
+
         if (proc == NULL) {
             break;
         }
@@ -1192,7 +1209,7 @@ ble_sm_enc_event_rx(uint16_t conn_handle, uint8_t evt_status, int encrypted)
     ble_hs_unlock();
 
     res.bonded = bonded;
-    ble_sm_process_result(conn_handle, &res);
+    ble_sm_process_result(conn_handle, &res, true);
 }
 
 void
@@ -1222,15 +1239,15 @@ ble_sm_retrieve_ltk(uint16_t ediv, uint64_t rand, uint8_t peer_addr_type,
     struct ble_store_key_sec key_sec;
     int rc;
 
-    /* Tell applicaiton to look up LTK by peer address and ediv/rand pair. */
+    /* Tell application to look up LTK by peer address and ediv/rand pair. */
     memset(&key_sec, 0, sizeof key_sec);
     key_sec.peer_addr.type = peer_addr_type;
     memcpy(key_sec.peer_addr.val, peer_addr, 6);
-    key_sec.ediv = ediv;
-    key_sec.rand_num = rand;
-    key_sec.ediv_rand_present = 1;
 
     rc = ble_store_read_our_sec(&key_sec, value_sec);
+    if (value_sec->ediv != ediv || value_sec->rand_num != rand) {
+        return BLE_HS_ENOENT;
+    }
     return rc;
 }
 
@@ -1413,7 +1430,7 @@ ble_sm_ltk_req_rx(const struct ble_hci_ev_le_subev_lt_key_req *ev)
         }
     }
 
-    ble_sm_process_result(conn_handle, &res);
+    ble_sm_process_result(conn_handle, &res, true);
 
     return 0;
 }
@@ -1737,10 +1754,10 @@ ble_sm_verify_auth_requirements(uint8_t cmd)
             return false;
         }
     }
-    /* Fail if Secure Connections level forces MITM protection and remote does not
+    /* Fail if security level forces MITM protection and remote does not
      * support it
      */
-    if (MYNEWT_VAL(BLE_SM_SC_LVL) >= 3 && !(cmd & BLE_SM_PAIR_AUTHREQ_MITM)) {
+    if (MYNEWT_VAL(BLE_SM_LVL) >= 3 && !(cmd & BLE_SM_PAIR_AUTHREQ_MITM)) {
         return false;
     }
     return true;
@@ -1823,7 +1840,7 @@ ble_sm_pair_req_rx(uint16_t conn_handle, struct os_mbuf **om,
         if (conn->bhc_flags & BLE_HS_CONN_F_MASTER) {
             res->sm_err = BLE_SM_ERR_CMD_NOT_SUPP;
             res->app_status = BLE_HS_SM_US_ERR(BLE_SM_ERR_CMD_NOT_SUPP);
-        } else if (MYNEWT_VAL(BLE_SM_SC_LVL) == 1) {
+        } else if (MYNEWT_VAL(BLE_SM_LVL) == 1) {
             res->sm_err = BLE_SM_ERR_CMD_NOT_SUPP;
             res->app_status = BLE_HS_SM_US_ERR(BLE_SM_ERR_CMD_NOT_SUPP);
         } else if (req->max_enc_key_size < BLE_SM_PAIR_KEY_SZ_MIN) {
@@ -1832,12 +1849,18 @@ ble_sm_pair_req_rx(uint16_t conn_handle, struct os_mbuf **om,
         } else if (req->max_enc_key_size > BLE_SM_PAIR_KEY_SZ_MAX) {
             res->sm_err = BLE_SM_ERR_INVAL;
             res->app_status = BLE_HS_SM_US_ERR(BLE_SM_ERR_INVAL);
-        } else if (MYNEWT_VAL(BLE_SM_SC_ONLY) && (req->max_enc_key_size != BLE_SM_PAIR_KEY_SZ_MAX)) {
-            /* Fail if Secure Connections Only mode is on and remote does not meet
-            * key size requirements - MITM was checked in last step
-            */
-            res->sm_err = BLE_SM_ERR_ENC_KEY_SZ;
-            res->app_status = BLE_HS_SM_US_ERR(BLE_SM_ERR_ENC_KEY_SZ);
+        } else if (MYNEWT_VAL(BLE_SM_SC_ONLY)) {
+            /* Fail if Secure Connections Only mode is on and remote does not
+             * meet key size requirements - MITM was checked in last step.
+             * Fail if SC is not supported by peer or key size is too small
+             */
+            if (!(req->authreq & BLE_SM_PAIR_AUTHREQ_SC)) {
+                res->sm_err = BLE_SM_ERR_AUTHREQ;
+                res->app_status = BLE_HS_SM_US_ERR(BLE_SM_ERR_AUTHREQ);
+            } else if (req->max_enc_key_size != BLE_SM_PAIR_KEY_SZ_MAX) {
+                res->sm_err = BLE_SM_ERR_ENC_KEY_SZ;
+                res->app_status = BLE_HS_SM_US_ERR(BLE_SM_ERR_ENC_KEY_SZ);
+            }
         } else if (!ble_sm_verify_auth_requirements(req->authreq)) {
             res->sm_err = BLE_SM_ERR_AUTHREQ;
             res->app_status = BLE_HS_SM_US_ERR(BLE_SM_ERR_AUTHREQ);
@@ -2051,6 +2074,7 @@ ble_sm_key_exch_success(struct ble_sm_proc *proc, struct ble_sm_result *res)
     res->app_status = 0;
     res->enc_cb = 1;
     res->bonded = bonded;
+    res->sm_err = BLE_SM_ERR_SUCCESS;
 }
 
 static void
@@ -2069,6 +2093,8 @@ ble_sm_key_exch_exec(struct ble_sm_proc *proc, struct ble_sm_result *res,
     uint8_t our_key_dist;
     struct os_mbuf *txom;
     const uint8_t *irk;
+    struct ble_store_gen_key gen_key;
+    int ltk_gen = 0;
     int rc;
 
     ble_sm_key_dist(proc, &init_key_dist, &resp_key_dist);
@@ -2086,15 +2112,37 @@ ble_sm_key_exch_exec(struct ble_sm_proc *proc, struct ble_sm_result *res,
             goto err;
         }
 
-        rc = ble_sm_gen_ltk(proc, enc_info->ltk);
-        if (rc != 0) {
-            os_mbuf_free_chain(txom);
-            goto err;
+        proc->our_keys.key_size = proc->key_size;
+
+        if (ble_hs_cfg.store_gen_key_cb) {
+            memset(&gen_key, 0, sizeof(gen_key));
+            rc = ble_hs_cfg.store_gen_key_cb(BLE_STORE_GEN_KEY_LTK, &gen_key,
+                                             proc->conn_handle);
+            if (rc == 0) {
+                /* Trim LRK to keysize */
+                memset(gen_key.ltk_periph + proc->key_size, 0,
+                       16 - proc->key_size);
+
+                proc->our_keys.ediv = gen_key.ediv;
+                proc->our_keys.rand_val = gen_key.rand;
+                memcpy(proc->our_keys.ltk, gen_key.ltk_periph, 16);
+
+                ltk_gen = 1;
+            }
         }
 
-        /* store LTK before sending since ble_sm_tx consumes tx mbuf */
-        memcpy(proc->our_keys.ltk, enc_info->ltk, 16);
-        proc->our_keys.key_size = proc->key_size;
+        if (!ltk_gen) {
+            rc = ble_sm_gen_ltk(proc, enc_info->ltk);
+            if (rc != 0) {
+                os_mbuf_free_chain(txom);
+                goto err;
+            }
+
+            /* store LTK before sending since ble_sm_tx consumes tx mbuf */
+            memcpy(proc->our_keys.ltk, enc_info->ltk, 16);
+        } else {
+            memcpy(enc_info->ltk, proc->our_keys.ltk, 16);
+        }
         proc->our_keys.ltk_valid = 1;
 
         rc = ble_sm_tx(proc->conn_handle, txom);
@@ -2110,20 +2158,25 @@ ble_sm_key_exch_exec(struct ble_sm_proc *proc, struct ble_sm_result *res,
             goto err;
         }
 
-        rc = ble_sm_gen_ediv(master_id);
-        if (rc != 0) {
-            os_mbuf_free_chain(txom);
-            goto err;
-        }
-        rc = ble_sm_gen_master_id_rand(master_id);
-        if (rc != 0) {
-            os_mbuf_free_chain(txom);
-            goto err;
-        }
+        if (!ltk_gen) {
+            rc = ble_sm_gen_ediv(master_id);
+            if (rc != 0) {
+                os_mbuf_free_chain(txom);
+                goto err;
+            }
+            rc = ble_sm_gen_master_id_rand(master_id);
+            if (rc != 0) {
+                os_mbuf_free_chain(txom);
+                goto err;
+            }
 
+            proc->our_keys.rand_val = master_id->rand_val;
+            proc->our_keys.ediv = master_id->ediv;
+        } else {
+            master_id->ediv = proc->our_keys.ediv;
+            master_id->rand_val = proc->our_keys.rand_val;
+        }
         proc->our_keys.ediv_rand_valid = 1;
-        proc->our_keys.rand_val = master_id->rand_val;
-        proc->our_keys.ediv = master_id->ediv;
 
         rc = ble_sm_tx(proc->conn_handle, txom);
         if (rc != 0) {
@@ -2188,14 +2241,29 @@ ble_sm_key_exch_exec(struct ble_sm_proc *proc, struct ble_sm_result *res,
             goto err;
         }
 
-        rc = ble_sm_gen_csrk(proc, sign_info->sig_key);
-        if (rc != 0) {
-            os_mbuf_free_chain(txom);
-            goto err;
+        if (ble_hs_cfg.store_gen_key_cb) {
+            memset(&gen_key, 0, sizeof(gen_key));
+            rc = ble_hs_cfg.store_gen_key_cb(BLE_STORE_GEN_KEY_CSRK, &gen_key,
+                                             proc->conn_handle);
+            if (rc == 0) {
+                memcpy(proc->our_keys.csrk, gen_key.csrk, 16);
+            }
+        } else {
+            rc = -1;
         }
 
+        if (rc != 0) {
+            rc = ble_sm_gen_csrk(proc, sign_info->sig_key);
+            if (rc != 0) {
+                os_mbuf_free_chain(txom);
+                goto err;
+            }
+
+            memcpy(proc->our_keys.csrk, sign_info->sig_key, 16);
+        } else {
+            memcpy(sign_info->sig_key, proc->our_keys.csrk, 16);
+        }
         proc->our_keys.csrk_valid = 1;
-        memcpy(proc->our_keys.csrk, sign_info->sig_key, 16);
 
         rc = ble_sm_tx(proc->conn_handle, txom);
         if (rc != 0) {
@@ -2421,6 +2489,7 @@ ble_sm_fail_rx(uint16_t conn_handle, struct os_mbuf **om,
         cmd = (struct ble_sm_pair_fail *)(*om)->om_data;
 
         res->app_status = BLE_HS_SM_PEER_ERR(cmd->reason);
+        res->sm_err =  cmd->reason;
     }
 }
 
@@ -2508,7 +2577,7 @@ ble_sm_pair_initiate(uint16_t conn_handle)
     }
 
     if (proc != NULL) {
-        ble_sm_process_result(conn_handle, &res);
+        ble_sm_process_result(conn_handle, &res, true);
     }
 
     return res.app_status;
@@ -2547,7 +2616,7 @@ ble_sm_slave_initiate(uint16_t conn_handle)
     ble_hs_unlock();
 
     if (proc != NULL) {
-        ble_sm_process_result(conn_handle, &res);
+        ble_sm_process_result(conn_handle, &res, true);
     }
 
     return res.app_status;
@@ -2601,7 +2670,7 @@ ble_sm_enc_initiate(uint16_t conn_handle, uint8_t key_size,
 
     ble_hs_unlock();
 
-    ble_sm_process_result(conn_handle, &res);
+    ble_sm_process_result(conn_handle, &res, true);
 
     return res.app_status;
 }
@@ -2639,7 +2708,8 @@ ble_sm_rx(struct ble_l2cap_chan *chan)
         memset(&res, 0, sizeof res);
 
         rx_cb(conn_handle, om, &res);
-        ble_sm_process_result(conn_handle, &res);
+        ble_sm_process_result(conn_handle, &res, op == BLE_SM_OP_PAIR_FAIL ?
+                              false : true);
         rc = res.app_status;
     } else {
         rc = BLE_HS_ENOTSUP;
@@ -2753,7 +2823,7 @@ ble_sm_inject_io(uint16_t conn_handle, struct ble_sm_io *pkey)
         return rc;
     }
 
-    ble_sm_process_result(conn_handle, &res);
+    ble_sm_process_result(conn_handle, &res, true);
     return res.app_status;
 }
 
@@ -2766,7 +2836,7 @@ ble_sm_connection_broken(uint16_t conn_handle)
     res.app_status = BLE_HS_ENOTCONN;
     res.enc_cb = 1;
 
-    ble_sm_process_result(conn_handle, &res);
+    ble_sm_process_result(conn_handle, &res, true);
 }
 
 int
